# Copyright (C) 2015 Chi-kwan Chan
# Copyright (C) 2015 Steward Observatory
#
# This file is part of mockservation.
#
# Mockservation is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Mockservation is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with mockservation.  If not, see <http://www.gnu.org/licenses/>.

from os   import path
from glob import glob

import h5py  as h5
import numpy as np

class Handle:
    def __init__(self, pattern, shape=None):
        ext = path.splitext(pattern)[1][1:]
        if ext != "h5" and ext != "hdf5":
            raise NameError("Fail to load unsupported pattern \"{}\"".
                            format(pattern))

        names = glob(pattern)
        self.files = [h5.File(name, "r") for name in names]
        self.files.sort(key=lambda f: abs(f['side'][-1]-f['side'][0]),
                        reverse=True)

        s = self.files[0]['images'].shape
        m = self.files[0].attrs['model']
        l = self.files[0].attrs['wavelength (cm)']
        for i, f in enumerate(self.files[1:]):
            fs = f['images'].shape
            fm = f.attrs['model']
            fl = f.attrs['wavelength (cm)']
            if (s != fs or m != fm or 1.0e-3 < abs(l-fl) / (l+fl)):
                raise NameError("\"{}\" and \"{}\" are for different models".
                                format(names[0], names[i]))

        if shape is None:
            self.shape = s[1:]
        elif isinstance(shape, int):
            self.shape = (shape, shape)
        else:
            self.shape = shape

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_val, trace):
        for f in self.files:
            f.close()

    def __iter__(self):
        self.time = self.files[0]['time']
        self.curr = 0
        return self

    def __next__(self):
        if self.curr >= len(self.time):
            raise StopIteration

        time = self.time[self.curr]
        data = [{'img':  f['images'][self.curr,:,:],
                 'size': self.size(f)} for f in self.files]
        self.curr += 1

        img  = self.rebin(data[0]['img'], self.shape)
        size = data[0]['size']
        for d in data[1:]:
            S = img.shape[0]
            s = int(round(S * d['size'] / size))
            l = (S - s) // 2
            u = (S - l)
            img[l:u,l:u] = self.rebin(d['img'], (s, s))

        return time, img

    def rebin(self, img, shape):
        if shape == img.shape:
            return img

        ratio = shape[0] / img.shape[0]
        if ratio != shape[1] / img.shape[1]:
            raise NameError("Rebin ratios are different")

        if ratio > 1:
            N,M =     shape
            n,m = img.shape
            return np.repeat(np.repeat(img, N//n, 0), M//m, 1)
        else:
            N,M = img.shape
            n,m =     shape
            return img.reshape(n,N//n,m,M//m).mean(3).mean(1)

    def size(self, f):
        s = f['side']
        return len(s) * (s[1] - s[0])
